//
// Parago Media GmbH & Co. KG, Jrgen Burle (jbaurle@parago.de)
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
// TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
// CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
//

using System;
using System.Text;
using System.Web;
using System.Web.Caching;
using System.Web.UI;

namespace Parago.Web {

	/// <summary>
	/// The BasePageSimple class exposes base functionality for web forms.
	/// </summary>
	public class BasePageSimple : Page {

		/// <summary>
		/// Loads view state and control state directly from cache 
		/// without using the CachePageStatePersister class.
		/// </summary>
		protected override object LoadPageStateFromPersistenceMedium() {

			// Get the cache key from the web form data
			string key=Request.Params["__VIEWSTATE_CACHEKEY"] as string;

			// Abort if cache key is not available or valid
			if(String.IsNullOrEmpty(key)||!key.StartsWith("VIEWSTATE_"))
				throw new ApplicationException("Missing vaild __VIEWSTATE_CACHEKEY");

			// Return cached view state and control state
			return Cache[key];

		}

		/// <summary>
		/// Saves view state and control state directly to cache without
		/// using the CachePageStatePersister class.
		/// </summary>
		protected override void SavePageStateToPersistenceMedium(object state) {

			StringBuilder key=new StringBuilder();

			// Generate a unique cache key
			key.Append("VIEWSTATE_")
				.Append(Session!=null?Session.SessionID:Guid.NewGuid().ToString())
				.Append("_")
				.Append(Request.RawUrl)
				.Append("_")
				.Append(DateTime.Now.Ticks.ToString());

			// Add view state and control state to cache
			Cache.Add(
				key.ToString(), 
				state, 
				null, 
				DateTime.Now.AddMinutes(15), 
				Cache.NoSlidingExpiration, 
				CacheItemPriority.Default, 
				null
			);

			// Register hidden field to store cache key in
			ClientScript.RegisterHiddenField("__VIEWSTATE_CACHEKEY", key.ToString());

		}

	}

}
